var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var gen_remix_exports = {};
__export(gen_remix_exports, {
  default: () => gen_remix_default
});
module.exports = __toCommonJS(gen_remix_exports);
var import_fs = __toESM(require("fs"));
var import_path = __toESM(require("path"));
var import_package = __toESM(require("../../package.json"));
function usage() {
  console.log(`
  Usage:
    $ gen-remix [options]

  Options:
    --config PATH       Config path (default: ./gen-remix.config.json)
    --packages PACKAGES List of packages to export
    --output PATH       Output path (default: ./app/remix.ts)
  `);
}
async function gen_remix_default() {
  let configPath = `gen-remix.config.json`;
  let outputPath = "./app/remix.ts";
  let packages = [];
  let capture = null;
  for (let i = 2; i < process.argv.length; i++) {
    const arg = process.argv[i];
    if (arg.startsWith("--")) {
      capture = null;
      switch (arg) {
        case "--config":
          configPath = process.argv[++i];
          break;
        case "--output":
          outputPath = process.argv[++i];
          break;
        case "--packages":
          capture = packages;
          break;
        default:
          usage();
          break;
      }
    } else if (capture) {
      capture.push(arg);
    }
  }
  let config;
  if (!import_fs.default.existsSync(configPath)) {
    if (!packages.length) {
      usage();
      return;
    }
    config = { exports: packages, overrides: {} };
  } else {
    config = JSON.parse(import_fs.default.readFileSync(configPath, "utf8"));
  }
  console.log("\u{1F680} Generating remix.ts exports...");
  const exports = {};
  for (const packageName of config.exports) {
    console.log(`\u{1F4E6} ${packageName}`);
    const packageJson2 = JSON.parse(import_fs.default.readFileSync(`node_modules/${packageName}/package.json`, "utf8"));
    exports[packageName] = {
      version: packageJson2.version,
      values: [],
      types: []
    };
    let typings = packageJson2.typings;
    if (!typings) {
      typings = import_path.default.dirname(packageJson2.main) + "/index.d.ts";
    }
    const content = import_fs.default.readFileSync(import_path.default.join(`node_modules/${packageName}`, typings), "utf8");
    const lines = content.replace(/\n/g, " ").replace(/(export\s+(type\s*)?{)/g, "\n$1").split("\n").filter((line) => line.trim().length > 0);
    for (let line of lines) {
      const match = line.match(/^export(\s+type)?\s*{(.*)}/);
      if (match) {
        const list = match[1] ? "types" : "values";
        const exportList = match[2].split(",").map((s) => s.trim()).filter(Boolean);
        for (let exportName of exportList) {
          const alias = exportName.match(/^(\w+)\s+as\s+(\w+)$/);
          if (alias) {
            exportName = alias[2];
          }
          exports[packageName][list].push(exportName);
        }
      }
    }
  }
  let exportOverrides = [];
  const hasOverrides = config.overrides && Object.keys(config.overrides).length;
  if (hasOverrides) {
    for (const sourcePackage of Object.keys(config.overrides)) {
      const sourceOverrides = config.overrides[sourcePackage];
      exportOverrides = [
        ...exportOverrides,
        ...Object.values(sourceOverrides).flatMap((overrides) => Object.entries(overrides)).map(([target, source]) => ({
          source,
          target,
          type: exports[sourcePackage].values.includes(source) ? "value" : "type"
        }))
      ];
    }
  }
  let output = `// @ts-nocheck
/* eslint-disable */
// This file was generated by gen-remix.ts v${import_package.default.version}
`;
  for (const packageName of Object.keys(exports)) {
    output += `
// ${packageName}@${exports[packageName].version}`;
  }
  if (hasOverrides) {
    output += `

// import overrides`;
    for (const sourcePackage of Object.keys(config.overrides)) {
      const sourceOverrides = config.overrides[sourcePackage];
      let imports = [];
      for (const originalPackage of Object.keys(sourceOverrides)) {
        imports = [
          ...imports,
          ...Object.values(sourceOverrides[originalPackage])
        ];
      }
      imports.sort();
      output += `
import {
${imports.map((i) => `  ${exports[sourcePackage].types.includes(i) ? "type " : ""}${i},`).join("\n")}
} from "${sourcePackage}";`;
    }
  }
  let allExports = [];
  output += `

// export packages`;
  for (const packageName of Object.keys(exports)) {
    console.log(`\u{1F4E6} ${packageName}`);
    const currValues = exports[packageName].values.filter((e) => !exportOverrides.some(({ target }) => e === target) && !allExports.includes(e));
    currValues.sort();
    allExports = [...allExports, ...currValues];
    output += `
export {
${currValues.map((e) => `  ${e},`).join("\n")}
} from "${packageName}";`;
    if (exports[packageName].types.length) {
      const currTypes = exports[packageName].types.filter((e) => !exportOverrides.some(({ target }) => e === target) && !allExports.includes(e));
      currTypes.sort();
      allExports = [...allExports, ...currTypes];
      output += `
export type {
${currTypes.map((e) => `  ${e},`).join("\n")}
} from "${packageName}";`;
    }
  }
  if (hasOverrides) {
    output += `

// export overrides`;
    const overrideValues = exportOverrides.filter(({ type }) => type === "value");
    if (overrideValues.length) {
      overrideValues.sort();
      output += `
export {
${overrideValues.map(({ source, target }) => `  ${source === target ? source : `${source} as ${target}`},`).join("\n")}
};`;
    }
    const overrideTypes = exportOverrides.filter(({ type }) => type === "type");
    if (overrideTypes.length) {
      overrideTypes.sort();
      output += `
export type {
${overrideTypes.map(({ source, target }) => `  ${source === target ? source : `${source} as ${target}`},`).join("\n")}
};`;
    }
  }
  console.log(`\u{1F4DD} Writing ${outputPath}...`);
  import_fs.default.writeFileSync(outputPath, output, "utf8");
  console.log("\u{1F3C1} Done!");
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
