var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var svg_sprite_exports = {};
__export(svg_sprite_exports, {
  default: () => svg_sprite_default
});
module.exports = __toCommonJS(svg_sprite_exports);
var fs = __toESM(require("node:fs"));
var path = __toESM(require("node:path"));
var import_svg_parser = __toESM(require("../libs/svg-parser"));
let rootFolder = "";
let outputFolder = "";
let template = null;
let namedComponents = false;
let componentsTemplate = null;
let component = "icon.tsx";
let sprite = "icon.svg";
let types = "";
function svg_sprite_default(args) {
  if (args.length >= 2) {
    rootFolder = normalizeFolder(args[0]);
    let outputPath = args[1];
    if (outputPath.endsWith(".tsx")) {
      outputFolder = path.dirname(outputPath);
      component = path.basename(outputPath);
    } else {
      outputFolder = normalizeFolder(outputPath);
    }
    for (let i = 2; i < args.length; i++) {
      if (args[i].startsWith("--template=")) {
        const templateFilename = args[i].substring("--template=".length);
        template = fs.readFileSync(templateFilename, "utf8");
      } else if (args[i].startsWith("--component=")) {
        sprite = args[i].substring("--component=".length);
      } else if (args[i].startsWith("--sprite=")) {
        sprite = args[i].substring("--sprite=".length);
      } else if (args[i].startsWith("--types=")) {
        types = args[i].substring("--types=".length);
      } else if (args[i] === "--components") {
        namedComponents = true;
      } else if (args[i].startsWith("--components-template=")) {
        const templateFilename = args[i].substring("--components-template=".length);
        componentsTemplate = fs.readFileSync(templateFilename, "utf8");
      }
    }
  } else {
    console.log(`
Usage: npx rmx-cli svg-sprite SOURCE_FOLDER OUTPUT_PATH
               [--sprite=FILENAME] [--types=FILENAME]
               [--components] [--template=TEMPLATE_FILE]

SOURCE_FOLDER: folder containing .svg files
OUTPUT_PATH: output path for sprite file and components

* If OUTPUT_PATH ends with .tsx, then use this as the base filename
  (default: icon.tsx)

--sprite=FILENAME: base filename of sprite file (default: icon.svg)
--types=FILENAME : base filename of IconType export file
                   if present, will not generate component file
--components     : generate named components for each icon
--template=TEMPLATE_FILE: use custom template file
`.trimStart());
    process.exit(1);
  }
  generateSprites(rootFolder);
}
const logs = [];
function log(...args) {
  logs.push(args);
}
function flushLogs() {
  if (logs.length === 0)
    return;
  logs.forEach((args) => console.log(...args));
  logs.length = 0;
}
function normalizeFolder(folder) {
  let fullPath = path.resolve(folder);
  const cwd = ensureSlash(process.cwd());
  let normalized = fullPath;
  if (normalized.startsWith(cwd)) {
    normalized = normalized.replace(cwd, "");
  }
  return normalized;
}
function ensureSlash(input) {
  if (!input.endsWith(path.sep)) {
    input = input + path.sep;
  }
  return input;
}
function generateSprites(folder) {
  const svgFiles = fs.readdirSync(folder).filter((file) => file.endsWith(".svg")).map((file) => path.join(folder, file));
  if (svgFiles.length > 0) {
    generateSprite(folder, svgFiles);
  }
  const folders = fs.readdirSync(folder, { withFileTypes: true }).filter((dirent) => dirent.isDirectory()).map((dirent) => path.join(folder, dirent.name));
  folders.forEach((folder2) => {
    generateSprites(folder2);
  });
}
function generateSprite(folder, files) {
  const spriteOutputFolder = folder.replace(rootFolder, outputFolder);
  const spriteOutput = path.join(spriteOutputFolder, path.basename(sprite));
  log(`\u{1F4DD} Generating sprite for ${folder}`);
  if (!fs.existsSync(spriteOutputFolder)) {
    fs.mkdirSync(spriteOutputFolder, { recursive: true });
  }
  const strip = true;
  const trim = false;
  let { svgElement } = import_svg_parser.default.iterateFiles(files, strip, trim);
  svgElement = import_svg_parser.default.wrapInSvgTag(svgElement);
  const exists = fs.existsSync(spriteOutput);
  let hasChanges = !exists;
  if (exists) {
    const existingSprite = fs.readFileSync(spriteOutput, "utf8");
    if (existingSprite !== svgElement) {
      hasChanges = true;
    }
  }
  if (hasChanges) {
    fs.writeFileSync(spriteOutput, svgElement, "utf8");
    flushLogs();
  }
  if (types) {
    generateTypesFile(spriteOutputFolder, files);
    return;
  }
  generateReactComponent(spriteOutputFolder, files);
}
function generateTypesFile(spriteOutputFolder, files) {
  let icons = files.map((file) => path.basename(file, ".svg"));
  let output = `export const iconNames = [
${icons.map((icon) => `  "${icon}",`).join("\n")}
] as const;

export type IconName = typeof iconNames[number];
`;
  icons.forEach((icon) => log(`\u2705 ${icon}`));
  let typesPath = path.join(spriteOutputFolder, types);
  const exists = fs.existsSync(typesPath);
  let hasChanges = !exists;
  if (exists) {
    const existing = fs.readFileSync(typesPath, "utf8");
    if (existing !== output) {
      hasChanges = true;
    }
  }
  if (hasChanges) {
    fs.writeFileSync(typesPath, output, "utf8");
    flushLogs();
  }
}
function generateReactComponent(spriteOutputFolder, files) {
  let icons = files.map((file) => path.basename(file, ".svg"));
  const spritePath = path.basename(component, ".tsx") + ".svg";
  let output = template != null ? template : `
import { type SVGProps } from "react";
import href from "./${spritePath}";
export { href };

export default function Icon({ icon, ...props}: SVGProps<SVGSVGElement> & { icon: IconName }) {
  return (
    <svg {...props}>
      <use href={\`\${href}#\${icon}\`} />
    </svg>
  );
}
`;
  if (!types) {
    output += `
export const iconNames = [
${icons.map((icon) => `  "${icon}",`).join("\n")}
] as const;
export type IconName = typeof iconNames[number];`;
  }
  icons.forEach((icon) => log(`\u2705 ${icon}`));
  if (namedComponents) {
    output += "\n";
    icons.forEach((icon) => {
      var _a;
      const componentName = icon.replace(/(^|-|_)([a-z0-9])/g, (g) => g.at(-1).toUpperCase());
      output += "\n";
      output += (_a = componentsTemplate == null ? void 0 : componentsTemplate.replace(/{{icon}}/g, icon).replace(/{{componentName}}/g, componentName)) != null ? _a : `export const ${componentName}Icon = (props: SVGProps<SVGSVGElement>) => <Icon icon="${icon}" {...props} />;`;
    });
  }
  const componentPath = path.join(spriteOutputFolder, component);
  output = output.trimStart();
  const exists = fs.existsSync(componentPath);
  let hasChanges = !exists;
  if (exists) {
    const existing = fs.readFileSync(componentPath, "utf8");
    if (existing !== output) {
      hasChanges = true;
    }
  }
  if (hasChanges) {
    fs.writeFileSync(componentPath, output, "utf8");
    flushLogs();
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
